@@ -4,7 +4,7 @@
 fn main() {
     #[cfg(windows)]
     if std::env::var("CARGO_CFG_TARGET_OS").unwrap_or_default() == "windows" {
-        winres::WindowsResource::new()
+        winresource::WindowsResource::new()
             .set_manifest_file("src/bin/edit/edit.exe.manifest")
             .set("FileDescription", "Microsoft Edit")
             .set("LegalCopyright", "© Microsoft Corporation. All rights reserved.")
