@@ -42,7 +42,7 @@ pub enum LocId {
     View,
     ViewFocusStatusbar,
     ViewWordWrap,
-    ViewDocumentPicker,
+    ViewGoToFile,
 
     // Help menu
     Help,
@@ -247,17 +247,17 @@ const S_LANG_LUT: [[&str; LangId::Count as usize]; LocId::Count as usize] = [
     ],
     // FileNew
     [
-        /* en      */ "New File…",
-        /* de      */ "Neue Datei…",
-        /* es      */ "Nuevo archivo…",
-        /* fr      */ "Nouveau fichier…",
-        /* it      */ "Nuovo file…",
-        /* ja      */ "新規ファイル…",
-        /* ko      */ "새 파일…",
-        /* pt_br   */ "Novo arquivo…",
-        /* ru      */ "Новый файл…",
-        /* zh_hans */ "新建文件…",
-        /* zh_hant */ "新增檔案…",
+        /* en      */ "New File",
+        /* de      */ "Neue Datei",
+        /* es      */ "Nuevo archivo",
+        /* fr      */ "Nouveau fichier",
+        /* it      */ "Nuovo file",
+        /* ja      */ "新規ファイル",
+        /* ko      */ "새 파일",
+        /* pt_br   */ "Novo arquivo",
+        /* ru      */ "Новый файл",
+        /* zh_hans */ "新建文件",
+        /* zh_hant */ "新增檔案",
     ],
     // FileOpen
     [
@@ -303,17 +303,17 @@ const S_LANG_LUT: [[&str; LangId::Count as usize]; LocId::Count as usize] = [
     ],
     // FileClose
     [
-        /* en      */ "Close Editor",
-        /* de      */ "Editor schließen",
-        /* es      */ "Cerrar editor",
-        /* fr      */ "Fermer l’éditeur",
-        /* it      */ "Chiudi editor",
-        /* ja      */ "エディターを閉じる",
-        /* ko      */ "편집기 닫기",
-        /* pt_br   */ "Fechar editor",
-        /* ru      */ "Закрыть редактор",
-        /* zh_hans */ "关闭编辑器",
-        /* zh_hant */ "關閉編輯器",
+        /* en      */ "Close File",
+        /* de      */ "Datei schließen",
+        /* es      */ "Cerrar archivo",
+        /* fr      */ "Fermer le fichier",
+        /* it      */ "Chiudi file",
+        /* ja      */ "ファイルを閉じる",
+        /* ko      */ "파일 닫기",
+        /* pt_br   */ "Fechar arquivo",
+        /* ru      */ "Закрыть файл",
+        /* zh_hans */ "关闭文件",
+        /* zh_hant */ "關閉檔案",
     ],
     // FileExit
     [
@@ -331,17 +331,17 @@ const S_LANG_LUT: [[&str; LangId::Count as usize]; LocId::Count as usize] = [
     ],
     // FileGoto
     [
-        /* en      */ "Go to Line/Column…",
-        /* de      */ "Gehe zu Zeile/Spalte…",
-        /* es      */ "Ir a línea/columna…",
-        /* fr      */ "Aller à la ligne/colonne…",
-        /* it      */ "Vai a riga/colonna…",
-        /* ja      */ "行/列へ移動…",
-        /* ko      */ "행/열로 이동…",
-        /* pt_br   */ "Ir para linha/coluna…",
-        /* ru      */ "Перейти к строке/столбцу…",
-        /* zh_hans */ "转到行/列…",
-        /* zh_hant */ "跳至行/列…",
+        /* en      */ "Go to Line:Column…",
+        /* de      */ "Gehe zu Zeile:Spalte…",
+        /* es      */ "Ir a línea:columna…",
+        /* fr      */ "Aller à la ligne:colonne…",
+        /* it      */ "Vai a riga:colonna…",
+        /* ja      */ "行:列へ移動…",
+        /* ko      */ "행:열로 이동…",
+        /* pt_br   */ "Ir para linha:coluna…",
+        /* ru      */ "Перейти к строке:столбцу…",
+        /* zh_hans */ "转到行:列…",
+        /* zh_hant */ "跳至行:列…",
     ],
 
     // Edit (a menu bar item)
@@ -513,19 +513,19 @@ const S_LANG_LUT: [[&str; LangId::Count as usize]; LocId::Count as usize] = [
         /* zh_hans */ "自动换行",
         /* zh_hant */ "自動換行",
     ],
-    // ViewDocumentPicker
+    // ViewGoToFile
     [
-        /* en      */ "Document Picker",
-        /* de      */ "Dokumentauswahl",
-        /* es      */ "Selector de documentos",
-        /* fr      */ "Sélecteur de documents",
-        /* it      */ "Selettore di documenti",
-        /* ja      */ "ドキュメントピッカー",
-        /* ko      */ "문서 선택기",
-        /* pt_br   */ "Seletor de documentos",
-        /* ru      */ "Выбор документа",
-        /* zh_hans */ "文档选择器",
-        /* zh_hant */ "文件選擇器",
+        /* en      */ "Go to File…",
+        /* de      */ "Gehe zu Datei…",
+        /* es      */ "Ir a archivo…",
+        /* fr      */ "Aller au fichier…",
+        /* it      */ "Vai al file…",
+        /* ja      */ "ファイルへ移動…",
+        /* ko      */ "파일로 이동…",
+        /* pt_br   */ "Ir para arquivo…",
+        /* ru      */ "Перейти к файлу…",
+        /* zh_hans */ "转到文件…",
+        /* zh_hant */ "跳至檔案…",
     ],
 
     // Help (a menu bar item)
@@ -661,7 +661,7 @@ const S_LANG_LUT: [[&str; LangId::Count as usize]; LocId::Count as usize] = [
     // LargeClipboardWarningLine2
     [
         /* en      */ "You copied {size} which may take a long time to share.",
-        /* de      */ "Sie haben {size} kopiert, das Weitergeben könnte lange dauern.",
+        /* de      */ "Sie haben {size} kopiert. Das Weitergeben könnte länger dauern.",
         /* es      */ "Copiaste {size}, lo que puede tardar en compartirse.",
         /* fr      */ "Vous avez copié {size}, ce qui peut être long à partager.",
         /* it      */ "Hai copiato {size}, potrebbe richiedere molto tempo per condividerlo.",
@@ -846,31 +846,31 @@ const S_LANG_LUT: [[&str; LangId::Count as usize]; LocId::Count as usize] = [
 
     // EncodingReopen
     [
-        /* en      */ "Reopen with encoding",
-        /* de      */ "Mit Kodierung erneut öffnen",
-        /* es      */ "Reabrir con codificación",
-        /* fr      */ "Rouvrir avec un encodage différent",
-        /* it      */ "Riapri con codifica",
-        /* ja      */ "指定エンコーディングで再度開く",
-        /* ko      */ "인코딩으로 다시 열기",
-        /* pt_br   */ "Reabrir com codificação",
-        /* ru      */ "Открыть снова с кодировкой",
-        /* zh_hans */ "使用编码重新打开",
-        /* zh_hant */ "使用編碼重新打開",
+        /* en      */ "Reopen with encoding…",
+        /* de      */ "Mit Kodierung erneut öffnen…",
+        /* es      */ "Reabrir con codificación…",
+        /* fr      */ "Rouvrir avec un encodage différent…",
+        /* it      */ "Riapri con codifica…",
+        /* ja      */ "指定エンコーディングで再度開く…",
+        /* ko      */ "인코딩으로 다시 열기…",
+        /* pt_br   */ "Reabrir com codificação…",
+        /* ru      */ "Открыть снова с кодировкой…",
+        /* zh_hans */ "使用编码重新打开…",
+        /* zh_hant */ "使用編碼重新打開…",
     ],
     // EncodingConvert
     [
-        /* en      */ "Convert to encoding",
-        /* de      */ "In Kodierung konvertieren",
-        /* es      */ "Convertir a otra codificación",
-        /* fr      */ "Convertir vers l’encodage",
-        /* it      */ "Converti in codifica",
-        /* ja      */ "エンコーディングを変換",
-        /* ko      */ "인코딩으로 변환",
-        /* pt_br   */ "Converter para codificação",
-        /* ru      */ "Преобразовать в кодировку",
-        /* zh_hans */ "转换为编码",
-        /* zh_hant */ "轉換為編碼",
+        /* en      */ "Convert to encoding…",
+        /* de      */ "In Kodierung konvertieren…",
+        /* es      */ "Convertir a otra codificación…",
+        /* fr      */ "Convertir vers l’encodage…",
+        /* it      */ "Converti in codifica…",
+        /* ja      */ "エンコーディングを変換…",
+        /* ko      */ "인코딩으로 변환…",
+        /* pt_br   */ "Converter para codificação…",
+        /* ru      */ "Преобразовать в кодировку…",
+        /* zh_hans */ "转换为编码…",
+        /* zh_hant */ "轉換為編碼…",
     ],
 
     // IndentationTabs
